cd('c:\Users\Anton\Documents\MATLAB\[Project][Vienna,2016-17][Anomaly_detection]\Data\Results');

% Create new directory for store results
newdir = '5_SVOC_S4';
mkdir(newdir)

cd('c:\Users\Anton\Documents\MATLAB\[Project][Vienna,2016-17][Anomaly_detection]');
clear; clc; close all



%% Load the data 

fprintf('Loading the data ...\n\n');

% data = load('C:\Users\Anton\Documents\CogSci\2.Year\Mobility\Project\Proj\eeg_data_new.txt');

% y = ts_lag( data( : , 1), 3);
% X = ts_lag( data( :, 2 : end), 3);

% Load data with artifact information
dat_help_1  = load('data\b000401_art.mat');
dat_help_2  = load('data\b000402_art.mat');

% Load EEG data
[~, rec1] = edfread('data\b000401.rec');
[~, rec2] = edfread('data\b000402.rec');

% Load hypnogram 
[~, hypno1] = edfread('data\b000401.edf');
[~, hypno2] = edfread('data\b000402.edf');

fprintf('\nData was loaded into workspace.\n\n');
pause(1);

% Select only EEG data
% Channels:
%           'Fp1-M2'  -  'Fp2-M1'
%           'C3-M2'   -  'C4-M1'
%           'O1-M2'   -  'O2-M1'


dat1 = rec1(1:6, :)';
dat2 = rec2(1:6, :)';



% Select only the information about the artifacts
artef1 = dat_help_1.data.art;
artef2 = dat_help_2.data.art;

% Clear rec (it is useless for now)
clearvars rec1 rec2 dat_help_1 dat_help_2 

%% Set initial parameters

fprintf('Setting initial parameters ... \n');
source = containers.Map([1, 2, 3, 4, 5, 6], {'Fp1M2', 'C3M2', 'O1M2', ...
                                'Fp2M1', 'C4M1', 'O2M1'});
Nvars = size(dat1, 2);  % number of features in the data
sampRate = 200;         % sampling rate
stage1 = 4;             % which stage we are interesting to ref data (see help select_data) 
stage2 = 4;             % which stage we are interesting to test data (see help select_data) 
show_lamb    = 1;       % Show the lambda plots ? , set 1 to YES, 0 to NO
show_lag     = 1;       % Show the lags plots ? , set 1 to YES, 0 to NO
show_hypno   = 1;       % Show the hypnogram plots ? , set 1 to YES, 0 to NO
alpha_quant  = 0.05;    % Set the treshold cut-off by alpha quantile of the distribution
fprintf('Done.\n');

%% Plot hypnograms 
if show_hypno
    subplot(2,1,1);
    plot_hypnogram(hypno1, 200, 'Hypnogram for the first night')
    subplot(2,1,2);
    plot_hypnogram(hypno2, 200, 'Hypnogram for the second night')
end

% pause();

%% Select the data and set bounds for data sets

data_ref    = select_data(dat1, hypno1, artef1, stage1, 5, 200);
data_test   = select_data(dat2, hypno2, artef2, stage2, 5, 200);

ref_bounds  = [2000, 22000];%size(data_ref,1)-1000];
test_bounds = [25000, 41000];
%% Testing the lambda values
fprintf('\nComputing Lasso Granger graphs for different lambdas... \n\n');
lambs = linspace(0.00001, 1, 25);

MSE_lamb = zeros(size(lambs,2), Nvars);
AIC_lamb = zeros(size(lambs,2), Nvars);
BIC_lamb = zeros(size(lambs,2), Nvars);

temp = data_ref(ref_bounds(1):ref_bounds(2), : );

for i = 1:Nvars
    
    for j = 1:size(lambs, 2)
    
    [~, ~, mse, aic, bic] = lassoGranger(temp', 3, lambs(j), 'l');
    MSE_lamb(j, i) = mse; 
    AIC_lamb(j, i) = aic;
    BIC_lamb(j, i) = bic;
    end
    
    temp = circshift(temp, [0 -1]);
    
end

fprintf('Done. \n\n');
%% Plot lambdas and find the best lambdas 
bic_show = 0;
Fig1_lamb = plot_lambdas(show_lamb, Nvars, lambs, MSE_lamb, AIC_lamb, BIC_lamb, bic_show);


% Find the lowest value of AIC and appropriate lamdas 
[~ , idx]   = min(AIC_lamb);

% Set mean of the lambdas as the final lambda 
%lamb        = mean(lambs(idx));
lamb = 0.15;
%% Testing for differnet model order (max time lag)

fprintf('\nComputing Lasso Granger graphs for different model order... \n\n');
lags = linspace(1, 10, 10);

MSE_lag = zeros(size(lags,2), Nvars);
AIC_lag = zeros(size(lags,2), Nvars);
BIC_lag = zeros(size(lags,2), Nvars);

temp = data_ref(ref_bounds(1):ref_bounds(2), : );    % Calculate for the same sample as lambdas

for i = 1:Nvars
    
    for j = 1:size(lags, 2)
    
    [~, ~, mse, aic, bic] = lassoGranger(temp', j, lamb, 'l');
    MSE_lag(j, i) = mse; 
    AIC_lag(j, i) = aic;
    BIC_lag(j, i) = bic;
    end
    
    temp = circshift(temp, [0 -1]);
    
end

fprintf('Done. \n\n');

%% Plot lags and find the best model order
BIC_show = 0;
Fig2_lags = plot_lags(show_lag, Nvars, lags, MSE_lag, AIC_lag, BIC_lag, BIC_show);

% Find the lowest value of AIC and appropriate model order 
[~ , idx]   = min(AIC_lag);


%% Parameters
time_L  = 4;      % time lag

% Define struct for betas collection
betas_ref   = struc_betas(); 
betas_test  = struc_betas();


%% Compute betas 

% Reference set 
fprintf('\nComputing values beta for REFERENCE set ...\n');

sample_ref = data_ref(ref_bounds(1):ref_bounds(2), : );

% Compute betas ref coeficients
betas_ref = betasREF_comp(sample_ref, betas_ref, time_L, lamb, Nvars);

fprintf('\nBetas values for reference set was calculated. \n\n');


%% Test set 

fprintf('\nComputing values beta for TEST set ...\n');
% x0 = betas_ref.coefs{1}; %% ACHTUNG - dat potom do loop 
eps = 10; 


% here compose for loop for all test betas

sample_test = data_test(test_bounds(1):test_bounds(2), :);


% Compute beta coeficients for test data
[betas_test] = betasTEST_comp(sample_test, betas_test, betas_ref, time_L, lamb, Nvars, eps);

fprintf('\nBetas values for test set was calculated. \n');

%% Compute anomaly treshold

wind_len    = 200;
start       =   1;

[anom_tresh] = atreshold_comp(wind_len, start, sample_ref, ...
    alpha_quant, time_L, Nvars, lamb, eps);
    


%% Slide the time windows to find anomalities in test data
%% Compute anomaly for the whole test set

anom_all = anomaly_comp(sample_ref, sample_test, cell2mat(betas_ref.coefs), ...
    cell2mat(betas_test.coefs), time_L);

if ~any(anom_all > anom_tresh)
    fprintf('\nWe did not find any anomalous source in the whole test set.\n');
else 
    ind = find(anom_all > anom_tresh);
    fprintf('\nWe found anomalous source/s in the whole test set.\n');
    fprintf('The anomalies was found in:\n');
    for i = 1:length(ind)
        fprintf('   %s - localization\n', source(ind(i)));
    end
end

%% Compute anomalies for the slided time windows
wind    = 200; % one window = 200HZ -> 1s
start   =   1;

[anom_test, loc, loc_source] = anom_find(wind, start, test_bounds, sample_ref, sample_test, ...
                                             betas_ref, betas_test, time_L, anom_tresh, Nvars);

 %% dalej                                        

x = linspace(1, size(sample_test, 1), size(sample_test, 1));

[Fig_final] = plot_anomaly(x, sample_test, loc, loc_source, Nvars, sampRate);


%% SAVE THE RESULTS

directory = strcat(pwd, '\Data\Results\', newdir, '\' , newdir);
save(directory, 'AIC_lag', 'AIC_lamb', 'anom_all', 'anom_test', 'anom_tresh', ...
    'betas_ref', 'betas_test', 'BIC_lag', 'BIC_lamb', 'eps', 'loc', 'loc_source', ...
    'MSE_lag', 'MSE_lamb');

lambplot = strcat(pwd, '\Data\Results\', newdir, '\' , 'Lambda_plot.png');
saveas(Fig1_lamb, lambplot);

lagsplot = strcat(pwd, '\Data\Results\', newdir, '\' , 'Lags_plot.png');
saveas(Fig2_lags, lagsplot);