function [anomaly] = anomaly_comp(data_ref, data_test, betas_ref, betas_test, L)
[Nobs , Nvars] = size(data_ref);

anomaly = zeros(1, Nvars);

    for i = 1:Nvars
        % Return row vector of std.s of the ref and test sets' variables 
        sd_ref = std(data_ref(:,i));              % std. dev of reference set
        sd_test = std(data_test(:,i));            % std. dev of test set 

        % Return beta coeficients of ref and test sets' in matrix format 
        b_ref = betas_ref(:,i);
        b_test = betas_test(:,i);

        % Return the mean of the concatenate lagged variables 
        [X_ref, ~] = ts_lag(data_ref, L, i); 
        [X_test, ~] = ts_lag(data_test, L, i); 

        mcon_ref  = mean(X_ref)';
        mcon_test = mean(X_test)';

        % Compute divergence 
        d_ab = divergence(sd_ref, sd_test, b_ref, b_test, mcon_ref);    % divergence 'ref' to 'test'
        d_ba = divergence(sd_test, sd_ref, b_test, b_ref, mcon_test);    % divergence 'test' to 'ref'          
        anomaly(i) = max([d_ab; d_ba]);
    end
    
end